/*
 * adc.c
 *
 * Copyright (c) 2012, ROSAND Technologies
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is part of the Contiki operating system.
 *
 */

#include <stdint.h>
#include <avr/io.h>
#include "dev/adc.h"

#define ADC_CYCLE               8

/*----------------------------------------------------------------*/
/*
 * We make 10 samples:
 *      - Ignore the max and min values,
 *      - Compute the mean of the remaining 8 readings and
 *      - Return the computed result
 *
 */
int16_t
adc(uint8_t channel)
{
  int16_t reading;
  int16_t max, min, res;
  uint8_t i;

  ADMUX = channel;                                          /* Use AREF, right adjusted reading, single ended input */
  min = 0x3FF;
  max = 0;
  res = 0;
  i = ADC_CYCLE + 2;
  ADCSRA = (1 << ADEN) | (1 << ADSC) | (1 << ADATE);        /* Enable ADC , start in free running mode */
  do {
    while(!(ADCSRA & (1 << ADIF)));                         /* Wait for conversion to complete */
    reading = ADCL;                                         /* Read the 8 LSB */
    reading |= (((uint8_t) ADCH) << 8);                     /* Then the 8 MSB */
    reading &= 0x3FF;                                       /* 10-bit decimation rate: 6 MSB are noise */
    if (min > reading) {
      min = reading;                                        /* Update min value */
    }
    if (max < reading) {
      max = reading;                                        /* Update max value */
    }
    res += reading;                                         /* Update accumulator */
    ADCSRA |= (1 << ADIF);                                  /* Clear the Interrupt Flag for a new conversion */
  } while (--i);
  ADCSRA = 0;                                               /* Turn off the ADC */

  res -= (min + max);                                       /* Ignore edge values */
  return res / ADC_CYCLE;                                   /* Take the mean */
}
